@@ -379,7 +379,7 @@ class SQLCompiler(object):
         features = self.connection.features
         compilers = [
             query.get_compiler(self.using, self.connection)
-            for query in self.query.combined_queries
+            for query in self.query.combined_queries if not query.is_empty()
         ]
         if not features.supports_slicing_ordering_in_compound:
             for query, compiler in zip(self.query.combined_queries, compilers):
@@ -874,7 +874,7 @@ class SQLCompiler(object):
             cursor = self.connection.cursor()
         try:
             cursor.execute(sql, params)
-        except Exception:
+        except Exception as original_exception:
             try:
                 # Might fail for server-side cursors (e.g. connection closed)
                 cursor.close()
@@ -883,7 +883,7 @@ class SQLCompiler(object):
                 # Python 2 doesn't chain exceptions. Remove this error
                 # silencing when dropping Python 2 compatibility.
                 pass
-            raise
+            raise original_exception
 
         if result_type == CURSOR:
             # Caller didn't specify a result_type, so just give them back the
